@@ -317,7 +317,7 @@ public class NtlmHttpURLConnection extends HttpURLConnection {
         while (entries.hasNext()) {
             Map.Entry entry = (Map.Entry) entries.next();
             if (key.equalsIgnoreCase((String) entry.getKey())) {
-                entry.setValue(value);
+                entry.setValue(values);
                 found = true;
                 break;
             }
