@@ -18,10 +18,7 @@
 
 package jcifs.netbios;
 
-import java.net.InetAddress;
-import java.net.DatagramSocket;
-import java.net.DatagramPacket;
-import java.net.UnknownHostException;
+import java.net.*;
 import java.io.IOException;
 import java.io.InterruptedIOException;
 import java.util.HashMap;
@@ -203,19 +200,24 @@ class NameServiceClient implements Runnable {
                     response.notify();
                 }
             }
+        } catch(SocketTimeoutException ste) {
         } catch( Exception ex ) {
             if( log.level > 2 )
                 ex.printStackTrace( log );
+        } finally {
             tryClose();
         }
     }
     void send( NameServicePacket request, NameServicePacket response,
                                             int timeout ) throws IOException {
         Integer nid = null;
-        int count = 0;
+        int max = NbtAddress.NBNS.length;
+
+        if (max == 0)
+            max = 1; /* No WINs, try only bcast addr */
 
         synchronized( response ) {
-            do {
+            while (max-- > 0) {
                 try {
                     synchronized( LOCK ) {
                         request.nameTrnId = getNextNameTrnId();
@@ -237,24 +239,25 @@ class NameServiceClient implements Runnable {
 
                     response.wait( timeout );
 
+                    if (response.received)
+                        return;
+
                 } catch( InterruptedException ie ) {
                 } finally {
                     responseTable.remove( nid );
                 }
 
-                if( !response.received &&
-                            NbtAddress.NBNS.length > 1 &&
-                            NbtAddress.isWINS( request.addr )) {
+                if (NbtAddress.isWINS( request.addr ) == false)
+                    break;
+
                                 /* Message was sent to WINS but
                                  * failed to receive response.
                                  * Try a different WINS server.
                                  */
-                    request.addr = NbtAddress.switchWINS();
-                    if( count == 0 ) {
-                        count = NbtAddress.NBNS.length - 1;
-                    }
-                }
-            } while( count-- > 0 );
+                if (request.addr == NbtAddress.getWINSAddress())
+                    NbtAddress.switchWINS();
+                request.addr = NbtAddress.getWINSAddress();
+            }
         }
     }
 
