@@ -151,6 +151,8 @@ abstract class ServerMessageBlock extends Response implements Request, SmbConsta
     static final byte SMB_COM_SESSION_SETUP_ANDX = (byte)0x73;
     static final byte SMB_COM_LOGOFF_ANDX        = (byte)0x74;
     static final byte SMB_COM_TREE_CONNECT_ANDX  = (byte)0x75;
+    static final byte SMB_COM_NT_TRANSACT        = (byte)0xA0;
+    static final byte SMB_COM_NT_TRANSACT_SECONDARY = (byte)0xA1;
     static final byte SMB_COM_NT_CREATE_ANDX     = (byte)0xA2;
 
     /*
@@ -194,6 +196,7 @@ abstract class ServerMessageBlock extends Response implements Request, SmbConsta
         flags2 = 0;
         errorCode = 0;
         received = false;
+        digest = null;
     }
     int writeString( String str, byte[] dst, int dstIndex ) {
         return writeString( str, dst, dstIndex, useUnicode );
@@ -246,7 +249,7 @@ Hexdump.hexdump( System.err, src, srcIndex, maxLen < 128 ? maxLen + 8 : 128 );
                         throw new RuntimeException( "zero termination not found" );
                     }
                 }
-                str = new String( src, srcIndex, len, "UnicodeLittle" );
+                str = new String( src, srcIndex, len, "UnicodeLittleUnmarked" );
             } else {
                 while( src[srcIndex + len] != (byte)0x00 ) {
                     len++;
@@ -311,7 +314,7 @@ Hexdump.hexdump( System.err, src, srcIndex, maxLen < 128 ? maxLen + 8 : 128 );
         if( wordCount != 0 ) {
             int n;
             if(( n = readParameterWordsWireFormat( buffer, bufferIndex )) != wordCount * 2 ) {
-                if( log.level > 2 ) {
+                if( log.level >= 5 ) {
                     log.println( "wordCount * 2=" + ( wordCount * 2 ) +
                             " but readParameterWordsWireFormat returned " + n );
                 }
@@ -325,7 +328,7 @@ Hexdump.hexdump( System.err, src, srcIndex, maxLen < 128 ? maxLen + 8 : 128 );
         if( byteCount != 0 ) {
             int n;
             if(( n = readBytesWireFormat( buffer, bufferIndex )) != byteCount ) {
-                if( log.level > 2 ) {
+                if( log.level >= 5 ) {
                     log.println( "byteCount=" + byteCount +
                             " but readBytesWireFormat returned " + n );
                 }
@@ -467,6 +470,12 @@ Hexdump.hexdump( System.err, src, srcIndex, maxLen < 128 ? maxLen + 8 : 128 );
             case SMB_COM_CREATE_DIRECTORY:
                 c = "SMB_COM_CREATE_DIRECTORY";
                 break;
+            case SMB_COM_NT_TRANSACT:
+                c = "SMB_COM_NT_TRANSACT";
+                break;
+            case SMB_COM_NT_TRANSACT_SECONDARY:
+                c = "SMB_COM_NT_TRANSACT_SECONDARY";
+                break;
             default:
                 c = "UNKNOWN";
         }
