@@ -10,8 +10,8 @@ interface SmbConstants {
     static final int DEFAULT_PORT = 445;
 
     static final int DEFAULT_MAX_MPX_COUNT = 10;
-    static final int DEFAULT_RESPONSE_TIMEOUT = 10000;
-    static final int DEFAULT_SO_TIMEOUT = 15000;
+    static final int DEFAULT_RESPONSE_TIMEOUT = 30000;
+    static final int DEFAULT_SO_TIMEOUT = 35000;
     static final int DEFAULT_RCV_BUF_SIZE = 60416;
     static final int DEFAULT_SND_BUF_SIZE = 16644;
     static final int DEFAULT_SSN_LIMIT = 250;
@@ -75,6 +75,27 @@ interface SmbConstants {
     static final int ATTR_NORMAL     = 0x080;
     static final int ATTR_TEMPORARY  = 0x100;
 
+    // access mask encoding
+    static final int FILE_READ_DATA        = 0x00000001; // 1
+    static final int FILE_WRITE_DATA       = 0x00000002; // 2
+    static final int FILE_APPEND_DATA      = 0x00000004; // 3
+    static final int FILE_READ_EA          = 0x00000008; // 4
+    static final int FILE_WRITE_EA         = 0x00000010; // 5
+    static final int FILE_EXECUTE          = 0x00000020; // 6
+    static final int FILE_DELETE           = 0x00000040; // 7
+    static final int FILE_READ_ATTRIBUTES  = 0x00000080; // 8
+    static final int FILE_WRITE_ATTRIBUTES = 0x00000100; // 9
+    static final int DELETE                = 0x00010000; // 16
+    static final int READ_CONTROL          = 0x00020000; // 17
+    static final int WRITE_DAC             = 0x00040000; // 18
+    static final int WRITE_OWNER           = 0x00080000; // 19
+    static final int SYNCHRONIZE           = 0x00100000; // 20
+    static final int GENERIC_ALL           = 0x10000000; // 28
+    static final int GENERIC_EXECUTE       = 0x20000000; // 29
+    static final int GENERIC_WRITE         = 0x40000000; // 30
+    static final int GENERIC_READ          = 0x80000000; // 31
+
+
     // flags for move and copy
     static final int FLAGS_TARGET_MUST_BE_FILE         = 0x0001; 
     static final int FLAGS_TARGET_MUST_BE_DIRECTORY    = 0x0002; 
@@ -103,9 +124,7 @@ interface SmbConstants {
     static final TimeZone TZ = TimeZone.getDefault();
 
     static final boolean USE_BATCHING = Config.getBoolean( "jcifs.smb.client.useBatching", true );
-    static final String OEM_ENCODING =
-                Config.getProperty( "jcifs.encoding",
-                        System.getProperty( "file.encoding" ));
+    static final String OEM_ENCODING = Config.getProperty( "jcifs.encoding", Config.DEFAULT_OEM_ENCODING );
     static final int DEFAULT_FLAGS2 =
                 FLAGS2_LONG_FILENAMES |
                 FLAGS2_EXTENDED_ATTRIBUTES |
