@@ -187,11 +187,12 @@ public class SmbTransport extends Transport implements SmbConstants {
                     NbtAddress.getLocalName() );
             out.write( sbuf, 0, ssp.writeWireFormat( sbuf, 0 ));
             if (readn( in, sbuf, 0, 4 ) < 4) {
+                try { socket.close(); } catch(IOException ioe) {};
                 throw new SmbException( "EOF during NetBIOS session request" );
             }
             switch( sbuf[0] & 0xFF ) {
                 case SessionServicePacket.POSITIVE_SESSION_RESPONSE:
-                    if( log.level > 2 )
+                    if( log.level >= 4 )
                         log.println( "session established ok with " + address );
                     return;
                 case SessionServicePacket.NEGATIVE_SESSION_RESPONSE:
@@ -229,7 +230,7 @@ public class SmbTransport extends Transport implements SmbConstants {
              * is a policy regarding which hosts a user can connect from. This
              * requires communicating over port 139 rather than 445.
              */
-            if (NETBIOS_HOSTNAME != null && NETBIOS_HOSTNAME.equals( "" ) == false) {
+            if (false && NETBIOS_HOSTNAME != null && NETBIOS_HOSTNAME.equals( "" ) == false) {
                 port = 139;
             }
             if (port == 139) {
@@ -252,9 +253,9 @@ public class SmbTransport extends Transport implements SmbConstants {
             int n = NEGOTIATE_REQUEST.encode( sbuf, 4 );
             Encdec.enc_uint32be( n & 0xFFFF, sbuf, 0 ); /* 4 byte ssn msg header */
 
-            if (log.level > 3) {
+            if (log.level >= 4) {
                 log.println( NEGOTIATE_REQUEST );
-                if (log.level > 5) {
+                if (log.level >= 6) {
                     Hexdump.hexdump( log, sbuf, 4, n );
                 }
             }
@@ -273,9 +274,9 @@ public class SmbTransport extends Transport implements SmbConstants {
             readn( in, sbuf, 4 + 32, size - 32 );
             resp.decode( sbuf, 4 );
 
-            if (log.level > 3) {
+            if (log.level >= 4) {
                 log.println( resp );
-                if (log.level > 5) {
+                if (log.level >= 6) {
                     Hexdump.hexdump( log, sbuf, 4, n );
                 }
             }
@@ -299,6 +300,9 @@ public class SmbTransport extends Transport implements SmbConstants {
         } catch( ConnectException ce ) {
             port = (port == 0 || port == DEFAULT_PORT) ? 139 : DEFAULT_PORT;
             negotiate( port, resp );
+        } catch( NoRouteToHostException nr ) {
+            port = (port == 0 || port == DEFAULT_PORT) ? 139 : DEFAULT_PORT;
+            negotiate( port, resp );
         }
 
         if( resp.dialectIndex > 10 ) {
@@ -354,7 +358,7 @@ public class SmbTransport extends Transport implements SmbConstants {
                                                    /* read smb header */
         if ((n = readn( in, sbuf, 4, 32 )) < 32)
             return null;
-        if (log.level > 2) {
+        if (log.level >= 4) {
             log.println( "New data read: " + this );
             jcifs.util.Hexdump.hexdump( log, sbuf, 4, 32 );
         }
@@ -401,12 +405,12 @@ public class SmbTransport extends Transport implements SmbConstants {
             ServerMessageBlock smb = (ServerMessageBlock)request;
             int n = smb.encode( BUF, 4 );
             Encdec.enc_uint32be( n & 0xFFFF, BUF, 0 ); /* 4 byte session message header */
-            if (log.level > 3) {
+            if (log.level >= 4) {
                 do {
                     log.println( smb );
                 } while (smb instanceof AndXServerMessageBlock &&
                         (smb = ((AndXServerMessageBlock)smb).andx) != null);
-                if (log.level > 5) {
+                if (log.level >= 6) {
                     Hexdump.hexdump( log, BUF, 4, n );
                 }
             }
@@ -463,6 +467,13 @@ public class SmbTransport extends Transport implements SmbConstants {
             if (digest != null && resp.errorCode == 0) {
                 digest.verify( BUF, 4, resp );
             }
+
+            if (log.level >= 4) {
+                log.println( response );
+                if (log.level >= 6) {
+                    Hexdump.hexdump( log, BUF, 4, size );
+                }
+            }
         }
     }
     protected void doSkip() throws IOException {
