@@ -57,6 +57,8 @@ public abstract class Transport implements Runnable {
     protected abstract void doRecv( Response response ) throws IOException;
     protected abstract void doSkip() throws IOException;
 
+    public Object setupDiscoLock = new Object();
+
     public void sendrecv( Request request,
                     Response response,
                     long timeout ) throws IOException {
@@ -101,7 +103,7 @@ public abstract class Transport implements Runnable {
                 synchronized (response_map) {
                     Response response = (Response)response_map.get( key );
                     if (response == null) {
-                        if (log.level > 2)
+                        if (log.level >= 4)
                             log.println( "Invalid key, skipping message" );
                         doSkip();
                     } else {
@@ -110,16 +112,16 @@ public abstract class Transport implements Runnable {
                         response_map.notifyAll();
                     }
                 }
-            } catch( IOException ex ) {
+            } catch( Exception ex ) {
                 String msg = ex.getMessage();
-                boolean hard = true;
+                boolean timeout = msg != null && msg.equals( "Read timed out" );
+                /* If just a timeout, try to disconnect gracefully
+                 */
+                boolean hard = timeout == false;
 
-                if (log.level > 2)
+                if (!timeout && log.level >= 3)
                     ex.printStackTrace( log );
 
-                if (msg != null && msg.equals( "Read timed out" )) {
-                    hard = false;
-                }
                 try {
                     disconnect( hard );
                 } catch( IOException ioe ) {
@@ -145,64 +147,85 @@ public abstract class Transport implements Runnable {
     protected abstract void doDisconnect( boolean hard ) throws IOException;
 
     public synchronized void connect( long timeout ) throws TransportException {
-        switch (state) {
-            case 0:
-                break;
-            case 3:
-                return; // already connected
-            case 4:
-                state = 0;
-                throw new TransportException( "Connection in error", te );
-            default:
-                throw new TransportException( "Invalid state: " + state );
-        }
-
-        state = 1;
-        te = null;
-        thread = new Thread( this, name );
-        thread.setDaemon( true );
-
-        synchronized (thread) {
-            thread.start();
-            try {
-                thread.wait( timeout );          /* wait for doConnect */
-            } catch( InterruptedException ie ) {
-                throw new TransportException( ie );
+        try {
+            switch (state) {
+                case 0:
+                    break;
+                case 3:
+                    return; // already connected
+                case 4:
+                    state = 0;
+                    throw new TransportException( "Connection in error", te );
+                default:
+                    TransportException te = new TransportException( "Invalid state: " + state );
+                    state = 0;
+                    throw te;
             }
 
-            switch (state) {
-                case 1: /* doConnect never returned */
-                    state = 0;
-                    thread = null;
-                    throw new TransportException( "Connection timeout" );
-                case 2:
-                    if (te != null) { /* doConnect throw Exception */
-                        state = 4;                        /* error */
+            state = 1;
+            te = null;
+            thread = new Thread( this, name );
+            thread.setDaemon( true );
+
+            synchronized (thread) {
+                thread.start();
+                thread.wait( timeout );          /* wait for doConnect */
+
+                switch (state) {
+                    case 1: /* doConnect never returned */
+                        state = 0;
                         thread = null;
-                        throw te;
-                    }
-                    state = 3;                         /* Success! */
-                    return;
-                default:
-                    te = new TransportException( "Invalid state: " + state );
+                        throw new TransportException( "Connection timeout" );
+                    case 2:
+                        if (te != null) { /* doConnect throw Exception */
+                            state = 4;                        /* error */
+                            thread = null;
+                            throw te;
+                        }
+                        state = 3;                         /* Success! */
+                        return;
+                }
+            }
+        } catch( InterruptedException ie ) {
+            state = 0;
+            thread = null;
+            throw new TransportException( ie );
+        } finally {
+            /* This guarantees that we leave in a valid state
+             */
+            if (state != 0 && state != 3) {
+                if (log.level >= 1)
+                    log.println("Invalid state: " + state);
+                state = 0;
+                thread = null;
             }
         }
     }
-    public synchronized void disconnect( boolean hard ) throws IOException {
-        switch (state) {
-            case 0: /* not connected - just return */
-                return;
-            case 3: /* connected - go ahead and disconnect */
-                if (response_map.size() != 0 && !hard) {
-                    break; /* outstanding requests */
+    public void disconnect( boolean hard ) throws IOException {
+        synchronized(setupDiscoLock) {
+            synchronized(this) {
+                switch (state) {
+                    case 0: /* not connected - just return */
+                        return;
+                    case 2:
+                        hard = true;
+                    case 3: /* connected - go ahead and disconnect */
+                        if (response_map.size() != 0 && !hard) {
+                            break; /* outstanding requests */
+                        }
+                        doDisconnect( hard );
+                    case 4: /* in error - reset the transport */
+                        thread = null;
+                        state = 0;
+                        break;
+                    default:
+                        if (log.level >= 1)
+                            log.println("Invalid state: " + state);
+                        thread = null;
+                        state = 0;
+                        break;
                 }
-                doDisconnect( hard );
-            case 4: /* in error - reset the transport */
-                thread = null;
-                state = 0;
-                break;
-            default:
-                throw new TransportException( "Invalid state: " + state );
+            }
         }
     }
     public void run() {
